@@ -7,6 +7,8 @@
 #include <stdio.h>
 #include <assert.h>
 
+#include "iostream.hpp"
+
 #include "asc_ctype.hpp"
 #include "string.hpp"
 #include "fstream.hpp"
@@ -75,14 +77,14 @@ namespace acommon {
     return *this;
   }
 
-  bool FStream::getline(String & str, char delem)
+  bool FStream::getline(String & str, char d)
   {
     str.clear();
     int c;
     bool prev_slash = false;
     while (true) {
-      c  = fgetc(file_);
-      if (c == EOF || (!prev_slash && static_cast<char>(c) == delem)) break;
+      c  = getc(file_);
+      if (c == EOF || (!prev_slash && static_cast<char>(c) == d)) break;
       str += static_cast<char>(c);
       prev_slash = c == '\\';
     }
@@ -90,6 +92,32 @@ namespace acommon {
     return true;
   }
 
+  char * FStream::getline(char * str, size_t s, char d)
+  {
+    int c = getc(file_);
+    if (c == EOF) return 0;
+    if (c == d) goto quit;
+    *str = static_cast<char>(c);
+    ++str;
+    s -= 2; // leave room for the null char
+    while (c = getc(file_), c != EOF && (c != d || str[-1] == '\\')) {
+      *str = static_cast<char>(c);
+      ++str;
+      --s;
+      if (s < 0) goto overflow;
+    }
+  quit:
+    *str = '\0';
+    return str;
+  overflow:
+    *str = '\0';
+    while (c = getc(file_), c != EOF) {
+      if (c == '\\') c = getc(file_); // ignore next char
+      else if (c == d) break;
+    }
+    return str;
+  }
+
   bool FStream::read(void * str, unsigned int n)
   {
     fread(str,1,n,file_);
