@@ -18,8 +18,20 @@ namespace acommon {
     char delem;
   public:
     IStream(char d = '\n') : delem(d) {}
-    bool getline(String & str) {return getline(str,delem);}
+    
+    char delim() const {return delem;}
+
+    // getline will read until delem, UNLESS it is protected by
+    // a backslash
     virtual bool getline(String &, char c) = 0;
+    bool getline(String & str) {return getline(str,delem);}
+
+    // Get a line ending in 'd' unless that d is preceded by
+    // a '\'.  If a line is longer than 's' than the rest of the
+    // line will be ignored. s must be >= 4
+    virtual char * getline(char *, size_t s, char d) = 0;
+    char * getline(char * str, size_t s) {return getline(str,s,delem);}
+
     virtual bool read(void *, unsigned int) = 0;
 
     virtual ~IStream() {}
