@@ -221,7 +221,7 @@ namespace aspeller {
 
     virtual bool   empty()    const {return !size();}
     virtual Size   size()     const = 0;
-    virtual StringEnumeration * elements() const {abort();} //FIXME
+    virtual StringEnumeration * elements() const {abort(); return 0; } //FIXME
 
     virtual Enum * detailed_elements() const = 0;
   };
@@ -248,10 +248,10 @@ namespace aspeller {
   // implemented in multi_ws.cc
   BasicMultiSet * new_default_multi_word_set();
 
-  // implemented in writable_ws.cc
+  // implemented in writable.cpp
   WritableWordSet * new_default_writable_word_set();
 
-  // implemented in writable_repl.cc
+  // implemented in writable.cpp
   WritableReplacementSet * new_default_writable_replacement_set();
 
   
