@@ -10,7 +10,7 @@
 #include "posib_err.hpp"
 #include "stack_ptr.hpp"
 #include "string.hpp"
-#include "string_buffer.hpp"
+#include "objstack.hpp"
 
 using namespace acommon;
 
@@ -249,6 +249,17 @@ namespace aspeller {
 	  out += lang->de_accent(in[i]);
       }
     }
+    void convert(ParmString in, char * out) const
+    {
+      if (!strip_accents) {
+        memcpy(out, in, in.size() + 1);
+      } else {
+        unsigned int i = 0;
+	for (; i != in.size(); ++i)
+	  out[i] = lang->de_accent(in[i]);
+        out[i] = '\0';
+      }
+    }
   };
 
   inline String to_lower(const Language & l, ParmString word) 
@@ -275,6 +286,15 @@ namespace aspeller {
       new_word.push_back(l.to_stripped(*i));
     }
   }
+  
+  inline void to_stripped(const Language & l, ParmString word, char * o)
+  {
+    for (const char * i = word; *i; ++i) {
+      if (l.special(*i).any()) ++i;
+      *o++ = l.to_stripped(*i);
+    }
+    *o = '\0';
+  }
 
   inline String to_stripped(const Language & l, ParmString word) 
   {
@@ -344,6 +364,8 @@ namespace aspeller {
   PosibErr<void> check_if_valid(const Language & l, ParmString word);
 
   PosibErr<Language *> new_language(Config &, ParmString lang = 0);
+
+  PosibErr<void> open_affix_file(Config &, FStream & o);
   
 }
 
