@@ -50,7 +50,7 @@ using std::pair;
 #include "fstream.hpp"
 #include "language.hpp"
 #include "stack_ptr.hpp"
-#include "string_buffer.hpp"
+#include "objstack.hpp"
 #include "vector.hpp"
 #include "vector_hash-t.hpp"
 #include "check_list.hpp"
@@ -122,7 +122,7 @@ static inline void mmap_free(char *, unsigned int)
 
 #endif
 
-namespace aspeller_default_readonly_ws {
+namespace {
 
   using namespace aspeller;
 
@@ -514,7 +514,7 @@ namespace aspeller_default_readonly_ws {
     
   static void soundslike_next(WordEntry * w)
   {
-    const char * cur = static_cast<const char *>(w->intr[0]);
+    const char * cur = (const char *)(w->intr[0]);
     w->word = cur;
     unsigned int len = strlen(cur);
     cur += len + 1;
@@ -577,12 +577,12 @@ namespace aspeller_default_readonly_ws {
 namespace aspeller {
 
   BasicWordSet * new_default_readonly_word_set() {
-    return new aspeller_default_readonly_ws::ReadOnlyWS();
+    return new ReadOnlyWS();
   }
   
 }
 
-namespace aspeller_default_readonly_ws {
+namespace {
 
   using namespace aspeller;
 
@@ -666,10 +666,10 @@ namespace aspeller_default_readonly_ws {
    
     bool use_jump_tables = use_soundslike || config.retrieve_bool("use-jump-tables");
 
-    CERR << (affix_compress ? "  AFFIX COMPRESS" : "")
-         << (use_soundslike ? "  USING SOUNDSLIKE" : "") 
-         << (use_jump_tables ? "  USING JUMP TABLES" : "")
-         << "\n";
+    //CERR << (affix_compress ? "  AFFIX COMPRESS" : "")
+    //     << (use_soundslike ? "  USING SOUNDSLIKE" : "") 
+    //     << (use_jump_tables ? "  USING JUMP TABLES" : "")
+    //     << "\n";
 
     assert(!(affix_compress && use_soundslike)); // FIXME: return error
 
@@ -857,11 +857,7 @@ namespace aspeller_default_readonly_ws {
         SoundMap::iterator j = sound_map.find(temp.c_str());
         if (j == sound_map.end()) {
           
-          SoundMap::value_type
-            to_insert(buf.alloc(temp.size()+1), SoundslikeList());
-          strncpy(const_cast<char *>(to_insert.first),
-                  temp.c_str(),
-                  temp.size() + 1);
+          SoundMap::value_type to_insert(buf.dup(temp), SoundslikeList());
           sound_map.insert(to_insert)->second.size = 1;
           
         } else {
@@ -1118,7 +1114,7 @@ namespace aspeller {
     if (!res) return res;
     lang.reset(res.data);
     lang->set_lang_defaults(config);
-    aspeller_default_readonly_ws::create(els,*lang,config);
+    create(els,*lang,config);
     return no_err;
   }
 }
